"use client";

import { useState, useEffect } from "react";
import { 
  collection, 
  addDoc, 
  updateDoc, 
  deleteDoc, 
  doc, 
  getDocs, 
  getDoc,
  onSnapshot,
  query,
  where 
} from "firebase/firestore";
import { db } from "@/lib/firebase";
import { Class } from "@/lib/types";
import { useActivity } from "./useActivity";

export const useClasses = () => {
  const [classes, setClasses] = useState<Class[]>([]);
  const [loading, setLoading] = useState(true);
  const [error, setError] = useState<string | null>(null);
  const { addActivity } = useActivity();

  useEffect(() => {
    const classesRef = collection(db, "classes");
    
    // Set up real-time listener
    const unsubscribe = onSnapshot(classesRef, async (snapshot) => {
      try {
        const classesList: Class[] = [];
        
       for (const classDoc of snapshot.docs) {
          const data = classDoc.data() as Omit<Class, "id" | "teacherName">;
          
          // Get teacher name if teacherId exists
          let teacherName = "";
          if (data.teacherId) {
            const teacherDoc = await getDoc(doc(db, "teachers", data.teacherId));
            if (teacherDoc.exists()) {
              const teacher = teacherDoc.data();
              teacherName = `${teacher.firstName} ${teacher.lastName}`;
            }
          }
          
          classesList.push({
            id: classDoc.id,
            ...data,
            teacherName
          });
        }

        setClasses(classesList);
        setLoading(false);
      } catch (err) {
        console.error("Error fetching classes:", err);
        setError("Failed to fetch classes");
        setLoading(false);
      }
    }, (err) => {
      console.error("Error setting up classes listener:", err);
      setError("Failed to listen to classes updates");
      setLoading(false);
    });

    // Cleanup subscription
    return () => unsubscribe();
  }, []);

  const addClass = async (classData: Omit<Class, "id" | "teacherName">) => {
    try {
      const docRef = await addDoc(collection(db, "classes"), classData);
      
      // Get teacher name
      let teacherName = "";
      if (classData.teacherId) {
        const teacherDoc = await getDoc(doc(db, "teachers", classData.teacherId));
        if (teacherDoc.exists()) {
          const teacher = teacherDoc.data();
          teacherName = `${teacher.firstName} ${teacher.lastName}`;
        }
      }
      
      const newClass = { 
        id: docRef.id, 
        ...classData,
        teacherName 
      };

       // Record activity
      await addActivity({
        type: "class_created",
        title: "New Class Created",
        subject: classData.name,
        details: `Level: ${classData.level}, Teacher: ${teacherName || "Unassigned"}`,
        entityId: docRef.id,
        date: new Date().toISOString()
      });
      
      return newClass;
    } catch (err) {
      console.error("Error adding class:", err);
      setError("Failed to add class");
      throw err;
    }
  };

  const updateClass = async (id: string, data: Partial<Class>) => {
    try {
      const classRef = doc(db, "classes", id);
      await updateDoc(classRef, data);
      
      // Get updated teacher name if teacherId is being updated
      let updatedData = { ...data };
      if (data.teacherId) {
        const teacherDoc = await getDoc(doc(db, "teachers", data.teacherId));
        if (teacherDoc.exists()) {
          const teacher = teacherDoc.data();
          updatedData.teacherName = `${teacher.firstName} ${teacher.lastName}`;
        }
      }
      
      return { id, ...updatedData };
    } catch (err) {
      console.error("Error updating class:", err);
      setError("Failed to update class");
      throw err;
    }
  };

  const deleteClass = async (id: string) => {
    try {
      await deleteDoc(doc(db, "classes", id));
    } catch (err) {
      console.error("Error deleting class:", err);
      setError("Failed to delete class");
      throw err;
    }
  };

  const getClassById = async (id: string) => {
    try {
      const classDoc = await getDoc(doc(db, "classes", id));
      if (classDoc.exists()) {
        const data = classDoc.data() as Omit<Class, "id" | "teacherName">;
        
        // Get teacher name
        let teacherName = "";
        if (data.teacherId) {
          const teacherDoc = await getDoc(doc(db, "teachers", data.teacherId));
          if (teacherDoc.exists()) {
            const teacher = teacherDoc.data();
            teacherName = `${teacher.firstName} ${teacher.lastName}`;
          }
        }
        
        return {
          id: classDoc.id,
          ...data,
          teacherName
        } as Class;
      }
      return null;
    } catch (err) {
      console.error("Error getting class:", err);
      throw err;
    }
  };

  const getClassesByTeacher = async (teacherId: string) => {
    try {
      const q = query(collection(db, "classes"), where("teacherId", "==", teacherId));
      const querySnapshot = await getDocs(q);
      
      return querySnapshot.docs.map(doc => ({
        id: doc.id,
        ...doc.data()
      })) as Class[];
    } catch (err) {
      console.error("Error getting classes by teacher:", err);
      throw err;
    }
  };

  return {
    classes,
    loading,
    error,
    addClass,
    updateClass,
    deleteClass,
    getClassById,
    getClassesByTeacher,
  };
};